; #############################################################################
; #############################################################################
; #############################################################################
TWI_INI:
	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	
	
	
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	out TWCR,temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_BUS_CHECK:
	in temp, PINC 
	com temp
	andi temp, 0b00110000
	tst temp	
	brne TWI_BUS_CHECK_ERROR
	sbi PORTD,LED_D_ROT		; ROT	
	ret
TWI_BUS_CHECK_ERROR:
	cbi PORTD,LED_D_ROT		; ROT	
	rcall wait_100ms
	rjmp TWI_BUS_CHECK
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; START-ROUTINE	(MASTER TRANSMITTER)										   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
; rcall TWI_MASTER_TRANSMITTER_START	; INP: temp1(SLA) out: FEHLER_CODE		
TWI_MASTER_TRANSMITTER_START:
	; 	FEHLER lschen	
	WDR
	clr FEHLER_CODE
	cbi PORTD,LED_D_GELB	; GELB 	
	; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, temp	
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08							; 0x08 START erfolgreich
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10							; 0x10 (repeated START)
	breq TWI_MASTER_TRANSMITTER_START_w
	; ERROR		
	cbi PORTD,LED_D_ROT		; ROT	
	ldi temp,(ERROR_TWI_MT_START)
	mov FEHLER_CODE, temp
	ret
TWI_MASTER_TRANSMITTER_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1								; 1 x links
	out TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	breq TWI_MASTER_TRANSMITTER_START_OK
	; ERROR		
	cbi PORTD,LED_D_ROT		; ROT	
	ldi temp,(ERROR_TWI_MT_SLA)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_OK:

ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; Ein Zeichen zum SLAVE zu senden										   	   
; 																			   
; INPUT:	temp1 <= DATEN													   
; 																			   
; 	rcall TWI_MASTER_TRANSMITTER_DATEN	; inp: temp1 OUT: FEHLER_CODE		   
TWI_MASTER_TRANSMITTER_DATEN:
	; FEHLERPRFUNG						
	tst FEHLER_CODE							; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATEN_w		; ja			
	ret										; zurck		
TWI_MASTER_TRANSMITTER_DATEN_w:
	; 	Load DATA						
	out TWDR, temp1							
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
	; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	breq TWI_MASTER_TRANSMITTER_DATEN_ok
	; ERROR								
	cbi PORTD,LED_D_ROT		; ROT	
	ldi temp,(ERROR_TWI_MT_DATA)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_ok:

ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; TWI/I2C START-ROUTINE														   
;																			   
; INPUT:	temp1 <= ADRESSE SLAVE											   
;																			   
; rcall TWI_MASTER_RECEIVER_START	; INP: temp1(SLA) OUT: FEHLER_CODE		   
TWI_MASTER_RECEIVER_START:
	; FEHLER lschen					
	WDR
	clr FEHLER_CODE
	cbi PORTD,LED_D_GELB	; GELB 	
	; SRAM lschen						
	STS	(adr_TWI_BEREICH_READ+0),NULL
	STS	(adr_TWI_BEREICH_READ+1),NULL
	STS	(adr_TWI_BEREICH_READ+2),NULL
	STS	(adr_TWI_BEREICH_READ+3),NULL
	STS	(adr_TWI_BEREICH_READ+4),NULL
	STS	(adr_TWI_BEREICH_READ+5),NULL
	STS	(adr_TWI_BEREICH_READ+6),NULL
	STS	(adr_TWI_BEREICH_READ+7),NULL
	STS	(adr_TWI_BEREICH_READ+8),NULL
	STS	(adr_TWI_BEREICH_READ+9),NULL
	STS	(adr_TWI_BEREICH_READ+10),NULL
	STS	(adr_TWI_BEREICH_READ+11),NULL
	STS	(adr_TWI_BEREICH_READ+12),NULL
	STS	(adr_TWI_BEREICH_READ+13),NULL
	STS	(adr_TWI_BEREICH_READ+14),NULL
	; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RECEIVER_START_w
	; ERROR												
	cbi PORTD,LED_D_ROT		; ROT	
	ldi temp,(ERROR_TWI_MR_START)
	mov FEHLER_CODE, temp
	ret
TWI_MASTER_RECEIVER_START_w:
	; Load SLA +"R" (R=1)								
	lsl  temp1	; 1 x links
	ori temp1, 0b00000001 
	out TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	breq TWI_MASTER_RECEIVER_ok
	; ERROR						
	cbi PORTD,LED_D_ROT		; ROT	
	ldi temp,(ERROR_TWI_MR_SLA)
	mov FEHLER_CODE, temp
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_oK:

ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! Zeichen empfangen werden sollen										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATENEMPFANG:
	; Fehlerprfung										
	tst FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_w
	ret
TWI_MASTER_RECEIVER_DATENEMPFANG_w:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_OK
	; ERROR
	cbi PORTD,LED_D_ROT		; ROT	
	clr temp1
	ldi temp,(ERROR_TWI_MR_DATA)
	mov FEHLER_CODE, temp	
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_OK:
	; DATEN einlesen							
	in temp1, TWDR						
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Zeichen empfangen werden soll, oder wenn								
; das letzte! Zeichen empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN:
	; Fehlerprfung										
	tst FEHLER_CODE
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w
	ret
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
	; TWINT abfragen									
	rcall TWI_TWINT_wait
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_OK	; ERROR
	; ERROR
	cbi PORTD,LED_D_ROT		; ROT	
	clr temp1
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov FEHLER_CODE, temp	
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_OK:
	; DATEN einlesen		
	in temp1, TWDR
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	out TWCR, temp	
	; kurze Pause
	rcall wait_1ms
	sbi PORTD,LED_D_GELB	; GELB 	
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; ERROR- Routine															   
; 																			   
TWI_ERROR:
	cbi PORTD,LED_D_ROT		; ROT	
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	rcall wait_100ms	
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)	; TWI STOP				
	out TWCR, temp
	rcall TWI_AUS
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
; #############################################################################
; #############################################################################
; #############################################################################
