;												
; DS1307 RTC ATmega8 / 1MHz						
; Bernhard.Erfurt@gmx.de						
; 11.11/2011									
;												
; LCD 16x2										
;												
; Bei Programmstart wird der RTC initialisiert,				
; die Zeit, Datum und Wochentag								
; im DS1307	gesetzt auf 31.12.2011 23:59:45					
; Der Ausgangspin des DS1307 liefert einen Sekundenimpuls.	
; Anschlieend wird ca. alle 100ms der RTC abgefragt		
; und das Ergebnis angezeigt								
;												
; Anschlussbelegung PINS						
;												
; PB0 ==> LCD									
; PB1 ==> LCD									
; PB2 ==> LCD									
; PB3 ==> LCD									
; PB4 ==> LCD									
; PB5 ==> LCD									
; PB6 ==>  										
; PB7 ==> 										
;												
; PC0 ==> 										
; PC1 ==> 										
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==> SDA									
; PC5 ==> SCL									
;												
; PD0 ==> 										
; PD1 ==> 										
; PD2 ==> 										
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==> LED GRN								
; PD6 ==> LED GELB								
; PD7 ==> LED ROT								
;												


; ATmega8 										
.include "m8def.inc"

; KONSTANTEN									
.equ TAKT		=1000000			; 1 MHz	

; TWI MASTER EINSTELLUNGEN						
.equ TWI_BIT_RATE			=18					; 18	; Bit-Rate							
.equ TWI_PRESCALER			=1					; 1		; Vorteiler: 0=1 1=4 2=16 3=64		
; FEHLER-CODES									
.equ ERROR_TWI_MT_START		= 10
.equ ERROR_TWI_MT_SLA		= 12
.equ ERROR_TWI_MT_DATA		= 14

.equ ERROR_TWI_MR_START		= 20
.equ ERROR_TWI_MR_SLA		= 21
.equ ERROR_TWI_MR_DATA		= 22
.equ ERROR_TWI_MR_DATAEND	= 23

; LEDS 											
.equ LED_D_GRUEN		= 5
.equ LED_D_GELB			= 6
.equ LED_D_ROT			= 7
; LCD											
.equ LCD_PORT				= PORTB
.equ LCD_D0					= 0
.equ LCD_D1					= 1
.equ LCD_D2					= 2
.equ LCD_D3					= 3
; LCD RS (Befehl/DATEN)							
.equ LCD_RS_PORT			= PORTB
.equ LCD_RS					= 4
; LCD_ENABLE									
.equ LCD_E_PORT				= PORTB
.equ LCD_E					= 5
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN					= 0x0060			; 0x0060 START Adresse 		

.equ adr_ss						=0x0060+10
.equ adr_mm						=0x0060+11
.equ adr_hh						=0x0060+12
.equ adr_TAG					=0x0060+13
.equ adr_WOCHENTAG				=0x0060+14			; 1= Mo 2= Di .....
.equ adr_MONAT					=0x0060+15
.equ adr_JAHR					=0x0060+16
.equ adr_MESZ					=0x0060+18			; DUMMY, wird nicht berechnet !


.equ adr_TWI_BEREICH_READ		= 0x0060+100			; BEREICH fr Master TWI/I2C	


; REGISTER										

.def DUMMY_R0	= R0	; wichtige Hilfsregister (nicht verwenden !)
.def DUMMY_R1	= R1	; wichtige Hilfsregister (nicht verwenden !)

.def NULL 	= R10
.def EINS 	= R11
.def VOLL	= R12

.def FEHLER_CODE= R14

.def temp 	= R16
.def temp1 	= R17
.def temp2 	= R18
.def temp3 	= R19
.def temp4 	= R20
.def temp5 	= R21
.def temp6 	= R22
.def temp7 	= R23
.def temp8 	= R24
.def temp9 	= R25

.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0
rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Bei unkorrekten Interrupt aufrufen erfolgt ein RESET ber WDR	
INTERRUPT_ERROR:
	wdr	
	cbi PORTD,LED_D_ROT		; LED ROT			
INTERRUPT_ERROR_SCHLEIFE:
	rjmp INTERRUPT_ERROR_SCHLEIFE

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM lschen										
SRAM_CLEAR:
	ldi ZL, LOW (adr_DATEN)
	ldi ZH, HIGH(adr_DATEN)
	clr temp
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND); Vergleich?		
	brlo SRAM_CLEAR_w	; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND)	; Vergleich?		
	brlo SRAM_CLEAR_w	; kleiner => SPRUNG	
	breq SRAM_CLEAR_w	; gleich  => SPRUNG	


; PORT B initialisieren								
	ldi temp, 0b11111111	; DISPLAY	
	out DDRB, temp
; PORT B setzen										
	out PORTB, temp				
; PORT C initialisieren								
	ldi temp, 0b00110000	
	out DDRC, temp
; PORT C setzen										
	out PORTC, temp				
; PORT D initialisieren								
	ldi temp, 0b11100000	; LEDs	
	out DDRD, temp
; PORT D setzen										
	out PORTD, temp				
; WDR_INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp
	; DATUM				
	ldi temp,31
	STS(adr_TAG),temp
	ldi temp,12
	STS(adr_MONAT),temp
	ldi temp,11
	STS(adr_JAHR), temp
	; ZEIT				
	ldi temp,23
	STS(adr_hh),temp
	ldi temp,59
	STS(adr_mm),temp
	ldi temp,45
	STS(adr_ss),temp
	; WOCHENTAG	1=Mo		
	ldi temp, 6
	STS(adr_WOCHENTAG),temp
	; MESZ					
	ldi temp, 0
	STS(adr_MESZ),temp
; LCD_INITIALISIERUNG		
	rcall LCD_INITIALISIERUNG
; ANZEIGE_BEGRUESSUNG								
	rcall ANZEIGE_BEGRUESSUNG
; LED-TEST											
	rcall LED_TEST
; TWI_INI											
	rcall TWI_INI
; interrupts freigeben								
	sei



	rcall TWI_BUS_CHECK
	rcall DS1307_UHR_STELLEN_INITIALISIEREN
	rcall ANZEIGE_ERROR	
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	rcall TWI_BUS_CHECK
	rcall DS1307_UHR_AUSLESEN
	rcall ANZEIGE_ZEIT_DATUM
	rcall ANZEIGE_ERROR	
	rcall PROGRAMM_CHECK
	rcall wait_100ms	
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "DS1307_TWI.asm"
.include "DS1307_TWI_DS1307.asm"
.include "DS1307_LCD.asm"
.include "DS1307_SONSTIGES.asm"
.include "DS1307_01MHZwait.asm"		
; ##############################################################################
; ##############################################################################
; ##############################################################################


TEXT_BEGRUESSUNG_1:
.db "DS1307 RTC",0

TEXT_BEGRUESSUNG_2:
.db "11.11.2011 V1a",0

